/*************************************************************************************************************************
									File Details
**************************************************************************************************************************
	File name 							: Parallel_FLASH.C
	
	Purpose								: This file has the driver functions for reading/writing/erasing/programming LDR 
										  file to parallel FLASH device

	Author								: Mitesh Moonat & Harshit Gaharwar
	
	VDSP version used for testing		: VisualDSP++ 5.0, Update 10
	
	Hardware used						: ADSP-21469, ADSP-21479, ADSP-21489, ADSP-21375, ADSP-21369 EZ-Kits 
										  and ADSP-21469, ADPS-21367, ADSP-21375 BUBs
	
	Connection details 					: See "main.C"
	
	Guidelines for change				: 1) No change required for a different processor.
	
										  2) For a new FLASH device, following changes are required in 
										     "Parallel_FLASH_Collect_Device_Info" function:
										     
										     - If the manufacturer ID is not listed in the Switch - Case structure, add 
										       new "case" for the new manufaturer ID. Then add new device specific 
										       information  of the new FLASH device in a new 
										       "if(parallel_flash_device_info->IDInfo.sDeviceID==<New Device ID>)" structure.
										       
										     - If the manufacturer ID is already listed in the Switch - Case structure, add 
										       new device specific information  of the new FLASH device in a new 
										       "if(parallel_flash_device_info->IDInfo.sDeviceID==<New Device ID>)" structure.
										       
**************************************************************************************************************************/

/*************************************************************************************************************************
									Include Files
*************************************************************************************************************************/
#include "Parallel_FLASH.h"


/*************************************************************************************************************************
									Global Variables
**************************************************************************************************************************/
PARALLEL_FLASH_DEVICE_INFO g_parallel_flash_device_info; //Stores all the device specific information of the FLASH device


/*************************************************************************************************************************
									Function Definitions
**************************************************************************************************************************/

/**************************************************************************************************************************
	Function name	:   Parallel_FLASH_Read_Identification
    Description		:   This function reads the manufacturing and device ID information from the FLASH
	Arguments		:	Parameter			 |	Description		 								| Valid values
						DEVICE_INFO			 |  Pointer to the s_id object to be initialized    | unsigned int 32 bit
	Return value	:   None
**************************************************************************************************************************/
void Parallel_FLASH_Read_Identification(s_id* DEVICE_INFO)
{
	
	//If a 16 bit FLASH is used in byte mode - A1 of the DSP connected to A0 of the FLASH
	#ifdef WORD_MEMORY_USED_FOR_BYTE
	
		//Send Auto Select command
		EP_Send_8bit(0xAAA,0xAA);
		EP_Send_8bit(0x555,0x55);
		EP_Send_8bit(0xAAA,0x90);
		
		//Extract the Manufaturer ID and Device ID info
		DEVICE_INFO->cManID 	= 		EP_Receive_8bit(0);
		DEVICE_INFO->sDeviceID 	=	 	EP_Receive_8bit(2);
		
	//If a 8 bit FLASH is used
	#else
		
		//Send Auto Select command
		EP_Send_8bit(0x5555,0xAA);
		EP_Send_8bit(0x2AAA,0x55);
		EP_Send_8bit(0x5555,0x90);
		
		//Extract the Manufaturer ID and Device ID info
		DEVICE_INFO->cManID 	= 		EP_Receive_8bit(0);
		DEVICE_INFO->sDeviceID 	=	 	EP_Receive_8bit(1);

	#endif
	
	//Send reset command to bring the FLASH back in read mode
	EP_Send_8bit(0,0xF0);
}


/**************************************************************************************************************************
	Function name	:   Parallel_FLASH_Collect_Device_Info
    Description		:   Collects device info such as manufaturer ID, device ID, sector map, device size etc
	Arguments		:	Parameter				 	|	Description		 									| Valid values
						parallel_flash_device_info 	|   Address of the device info object to be initialized	| unsigned int 32 bit
	Return value	:   None
**************************************************************************************************************************/
void Parallel_FLASH_Collect_Device_Info(PARALLEL_FLASH_DEVICE_INFO* parallel_flash_device_info)
{

	//Read device ID and manufaturer ID
	Parallel_FLASH_Read_Identification(&(parallel_flash_device_info->IDInfo));

	//Manufaturer name and Device name
	switch(parallel_flash_device_info->IDInfo.cManID)
	{
		//Numonyx/ST Micro
		case 0x20:
		default:
				//Initialize the manufaturer name here
				strcpy(parallel_flash_device_info->ManName,"Numonyx/ST Micro"); 

				//Compare the device ID to check if the device is M29W320EB - 214xx EZ-Kits
				if(parallel_flash_device_info->IDInfo.sDeviceID==0x57)			
				{
					//Initialize the complete device name here
					strcpy(parallel_flash_device_info->DeviceName,"M29W320EB - 32 Mbit 3V supply Flash memory");

					//M29W320EB does not have uniform sectors, so "false"
					parallel_flash_device_info->SectorInfo.bUniformSectors			=	false;
					
					//Initialzing the main sector info here
					parallel_flash_device_info->SectorInfo.uiBytesPerSector			=	65536;
					parallel_flash_device_info->SectorInfo.uiTotalSectors			=	63;
					
				
					//Parameter blocks are at the bottom (starts from first location), so false
					parallel_flash_device_info->SectorInfo.bTopParameterBlock		=	false;
					
					//Initializing the parameter block info here
					parallel_flash_device_info->SectorInfo.uiBytesPerParameterBlock	=	8192;
					parallel_flash_device_info->SectorInfo.uiTotalParameterBlocks	=	8;
					
					//Initialize total device size in bytes
					parallel_flash_device_info->SectorInfo.uiTotalBytes=				33554432;
			
					//M29W320EB is a 16 bit device used in byte mode, so "true"
					parallel_flash_device_info->CommandSet.bWordAsByte				=	true;
				
					
					//Initialize Sector Erase Address and Data Sequence here
					parallel_flash_device_info->CommandSet.uiSectorEraseAddSeq[0]	=	0xAAA;
					parallel_flash_device_info->CommandSet.uiSectorEraseDatSeq[0]	=	0xAA;
				
					parallel_flash_device_info->CommandSet.uiSectorEraseAddSeq[1]	=	0x555;
					parallel_flash_device_info->CommandSet.uiSectorEraseDatSeq[1]	=	0x55;
				
					parallel_flash_device_info->CommandSet.uiSectorEraseAddSeq[2]	=	0xAAA;
					parallel_flash_device_info->CommandSet.uiSectorEraseDatSeq[2]	=	0x80;
				
					parallel_flash_device_info->CommandSet.uiSectorEraseAddSeq[3]	=	0xAAA;
					parallel_flash_device_info->CommandSet.uiSectorEraseDatSeq[3]	=	0xAA;
				
					parallel_flash_device_info->CommandSet.uiSectorEraseAddSeq[4]	=	0x555;
					parallel_flash_device_info->CommandSet.uiSectorEraseDatSeq[4]	=	0x55;
				
					parallel_flash_device_info->CommandSet.uiSectorEraseDatSeq[5]	=	0x30;
				
					
				
					//Initialize Byte Program Address and Data Sequence here
					parallel_flash_device_info->CommandSet.uiByteProgramAddSeq[0]	=	0xAAA;
					parallel_flash_device_info->CommandSet.uiByteProgramDatSeq[0]	=	0xAA;
			
					parallel_flash_device_info->CommandSet.uiByteProgramAddSeq[1]	=	0x555;
					parallel_flash_device_info->CommandSet.uiByteProgramDatSeq[1]	=	0x55;
			
					parallel_flash_device_info->CommandSet.uiByteProgramAddSeq[2]	=	0xAAA;
					parallel_flash_device_info->CommandSet.uiByteProgramDatSeq[2]	=	0xA0;
		
				}
				
				//Compare the device ID to check if the device is M29W040B - External
				if(parallel_flash_device_info->IDInfo.sDeviceID==0xE3)
				{
					//Initialize the complete device name here
					strcpy(parallel_flash_device_info->DeviceName,"M29W040B - 4 Mbit Uniform Block Low Voltage Single Supply Flash Memory");

					//M29W320EB does have uniform sectors of 64 KB size, so "true"
					parallel_flash_device_info->SectorInfo.bUniformSectors			=	true;
					
					//Initialzing the main sector info here
					parallel_flash_device_info->SectorInfo.uiBytesPerSector			=	65536;
					parallel_flash_device_info->SectorInfo.uiTotalSectors			=	8;
									
					//M29W320EB is an 8 bit device used in byte mode, so "false"
					parallel_flash_device_info->CommandSet.bWordAsByte				=	false;
				
					
					//Initializing Sector Erase Address and Data Sequence here
					parallel_flash_device_info->CommandSet.uiSectorEraseAddSeq[0]	=	0x555;
					parallel_flash_device_info->CommandSet.uiSectorEraseDatSeq[0]	=	0xAA;
				
					parallel_flash_device_info->CommandSet.uiSectorEraseAddSeq[1]	=	0x2AA;
					parallel_flash_device_info->CommandSet.uiSectorEraseDatSeq[1]	=	0x55;
				
					parallel_flash_device_info->CommandSet.uiSectorEraseAddSeq[2]	=	0x555;
					parallel_flash_device_info->CommandSet.uiSectorEraseDatSeq[2]	=	0x80;
				
					parallel_flash_device_info->CommandSet.uiSectorEraseAddSeq[3]	=	0x555;
					parallel_flash_device_info->CommandSet.uiSectorEraseDatSeq[3]	=	0xAA;
				
					parallel_flash_device_info->CommandSet.uiSectorEraseAddSeq[4]	=	0x2AA;
					parallel_flash_device_info->CommandSet.uiSectorEraseDatSeq[4]	=	0x55;
				
					parallel_flash_device_info->CommandSet.uiSectorEraseDatSeq[5]	=	0x30;
				
					
				
					//Initializing Byte Program Address and Data Sequence here
					parallel_flash_device_info->CommandSet.uiByteProgramAddSeq[0]	=	0x555;
					parallel_flash_device_info->CommandSet.uiByteProgramDatSeq[0]	=	0xAA;
			
					parallel_flash_device_info->CommandSet.uiByteProgramAddSeq[1]	=	0x2AA;
					parallel_flash_device_info->CommandSet.uiByteProgramDatSeq[1]	=	0x55;
			
					parallel_flash_device_info->CommandSet.uiByteProgramAddSeq[2]	=	0x555;
					parallel_flash_device_info->CommandSet.uiByteProgramDatSeq[2]	=	0xA0;
		
				}
				break;
				
		//AMD
		case 0x01:
				
				//Initialize the manufaturer name here
				strcpy(parallel_flash_device_info->ManName,"AMD");

				//Compare the device ID to check if the device is AM29LV081 - 21262/21364/21369/21375 EZ-Kits
				if(parallel_flash_device_info->IDInfo.sDeviceID==0x38)
				{
					//Initialize the complete device name here
					strcpy(parallel_flash_device_info->DeviceName,"AM29LV081 - 8 Megabit Uniform Sector Flash Memory");

					//AM29LV081 does have uniform sectors of 64 KB size, so "true"
					parallel_flash_device_info->SectorInfo.bUniformSectors			=	true;
					
					//Initialzing the main sector info here
					parallel_flash_device_info->SectorInfo.uiBytesPerSector			=	65536;
					parallel_flash_device_info->SectorInfo.uiTotalSectors			=	16;
					
					//AM29LV081 is an 8 bit device used in byte mode, so "false"					
					parallel_flash_device_info->CommandSet.bWordAsByte				=	false;
				
					
					//Initializing Sector Erase Address and Data Sequence here
					parallel_flash_device_info->CommandSet.uiSectorEraseAddSeq[0]	=	0;
					parallel_flash_device_info->CommandSet.uiSectorEraseDatSeq[0]	=	0xAA;
				
					parallel_flash_device_info->CommandSet.uiSectorEraseAddSeq[1]	=	0;
					parallel_flash_device_info->CommandSet.uiSectorEraseDatSeq[1]	=	0x55;
				
					parallel_flash_device_info->CommandSet.uiSectorEraseAddSeq[2]	=	0;
					parallel_flash_device_info->CommandSet.uiSectorEraseDatSeq[2]	=	0x80;
				
					parallel_flash_device_info->CommandSet.uiSectorEraseAddSeq[3]	=	0;
					parallel_flash_device_info->CommandSet.uiSectorEraseDatSeq[3]	=	0xAA;
				
					parallel_flash_device_info->CommandSet.uiSectorEraseAddSeq[4]	=	0;
					parallel_flash_device_info->CommandSet.uiSectorEraseDatSeq[4]	=	0x55;
				
					parallel_flash_device_info->CommandSet.uiSectorEraseDatSeq[5]	=	0x30;
				
					
					//Iniitalizing Byte Program Address and Data Sequence here
					parallel_flash_device_info->CommandSet.uiByteProgramAddSeq[0]	=	0;
					parallel_flash_device_info->CommandSet.uiByteProgramDatSeq[0]	=	0xAA;
			
					parallel_flash_device_info->CommandSet.uiByteProgramAddSeq[1]	=	0;
					parallel_flash_device_info->CommandSet.uiByteProgramDatSeq[1]	=	0x55;
			
					parallel_flash_device_info->CommandSet.uiByteProgramAddSeq[2]	=	0;
					parallel_flash_device_info->CommandSet.uiByteProgramDatSeq[2]	=	0xA0;
		
				}
				
				break;
				
		//SST
		case 0xBF:
		
				//Initialize the manufaturer name here
				strcpy(parallel_flash_device_info->ManName,"SST");

				//Compare the device ID to check if the device is SST39LF040 - External
				if(parallel_flash_device_info->IDInfo.sDeviceID==0xD7)
				{
					//Initialize the complete device name here
					strcpy(parallel_flash_device_info->DeviceName,"SST39LF040 - 4 Mbit (x8) Multi-Purpose Flash");

					//SST39LF040 does have uniform sectors of 4 KB size, so "true"
					parallel_flash_device_info->SectorInfo.bUniformSectors			=	true;
					
					//Initialzing the main sector info here
					parallel_flash_device_info->SectorInfo.uiBytesPerSector			=	4096;
					parallel_flash_device_info->SectorInfo.uiTotalSectors			=	128;
					
					//AM29LV081 is an 8 bit device used in byte mode, so "false"
					parallel_flash_device_info->CommandSet.bWordAsByte				=	false;
				
					
					//Initialize Sector Erase Address and Data Sequence here
					parallel_flash_device_info->CommandSet.uiSectorEraseAddSeq[0]	=	0x5555;
					parallel_flash_device_info->CommandSet.uiSectorEraseDatSeq[0]	=	0xAA;
				
					parallel_flash_device_info->CommandSet.uiSectorEraseAddSeq[1]	=	0x2AAA;
					parallel_flash_device_info->CommandSet.uiSectorEraseDatSeq[1]	=	0x55;
				
					parallel_flash_device_info->CommandSet.uiSectorEraseAddSeq[2]	=	0x5555;
					parallel_flash_device_info->CommandSet.uiSectorEraseDatSeq[2]	=	0x80;
				
					parallel_flash_device_info->CommandSet.uiSectorEraseAddSeq[3]	=	0x5555;
					parallel_flash_device_info->CommandSet.uiSectorEraseDatSeq[3]	=	0xAA;
				
					parallel_flash_device_info->CommandSet.uiSectorEraseAddSeq[4]	=	0x2AAA;
					parallel_flash_device_info->CommandSet.uiSectorEraseDatSeq[4]	=	0x55;
				
					parallel_flash_device_info->CommandSet.uiSectorEraseDatSeq[5]	=	0x30;
				
					
				
					//Initialize Byte Program Address and Data Sequence here
					parallel_flash_device_info->CommandSet.uiByteProgramAddSeq[0]	=	0x5555;
					parallel_flash_device_info->CommandSet.uiByteProgramDatSeq[0]	=	0xAA;
			
					parallel_flash_device_info->CommandSet.uiByteProgramAddSeq[1]	=	0x2AAA;
					parallel_flash_device_info->CommandSet.uiByteProgramDatSeq[1]	=	0x55;
			
					parallel_flash_device_info->CommandSet.uiByteProgramAddSeq[2]	=	0x5555;
					parallel_flash_device_info->CommandSet.uiByteProgramDatSeq[2]	=	0xA0;
				}
			break;
	}
}

/**************************************************************************************************************************
	Function name	:   Parallel_FLASH_Print_Device_Info
    Description		:   Prints device info such as manufaturer ID, device ID, sector map, page size, device size etc
	Arguments		:	Parameter				 	|	Description		 								| Valid values
						parallel_flash_device_info 	|  Address of the device info object to be printed	| unsigned int 32 bit
	Return value	:   None
**************************************************************************************************************************/
void Parallel_FLASH_Print_Device_Info(PARALLEL_FLASH_DEVICE_INFO* parallel_flash_device_info)
{

	#if !defined(__2137x__)
	
		printf("\n***************************************************************************************************************");
		printf("\n						 				 Parallel FLASH Device Information							 							 ");
		printf("\n***************************************************************************************************************");
		printf("\n");
		printf("\nManufacturer				: %s",	parallel_flash_device_info->ManName);
		printf("\nDevice name					: %s",parallel_flash_device_info->DeviceName);
		printf("\nUniform Sectors ?			: %d",parallel_flash_device_info->SectorInfo.bUniformSectors);
		printf("\nBytes per sector			: %d",parallel_flash_device_info->SectorInfo.uiBytesPerSector);
		printf("\nTotal sectors				: %d",parallel_flash_device_info->SectorInfo.uiTotalSectors);
	
		if(parallel_flash_device_info->SectorInfo.bUniformSectors==false)
		{
			printf("\nTop parameter block?		: %d",parallel_flash_device_info->SectorInfo.bTopParameterBlock);
			printf("\nBytes per  parameter block?	: %d",parallel_flash_device_info->SectorInfo.uiBytesPerParameterBlock);
			printf("\nTotal Parameter Blocks?		: %d",parallel_flash_device_info->SectorInfo.uiTotalParameterBlocks);
			
		}
		printf("\n");
		printf("\n***************************************************************************************************************");
		printf("\n");
		
	#endif
}


/**************************************************************************************************************************
	Function name	:   Parallel_FLASH_Sector_Erase
    Description		:   This function erases a particular sector of the FLASH
	Arguments		:	Parameter					 |	Description		 								| Valid values
						sector_address		 		 |  Any byte address belonging to the sector	 	| unsigned int 32 bit
						parallel_flash_device_info   |  Struture containing the FLASH device information| Pointer to device info
	Return value	:   =0 for fail, = 1 for pass
**************************************************************************************************************************/
int Parallel_FLASH_Sector_Erase(unsigned int sector_address, PARALLEL_FLASH_DEVICE_INFO* parallel_flash_device_info)
{

	//Temporary variable
	unsigned char ucTemp;
	
	//Send Block Erase Command Sequence
	EP_Send_8bit(parallel_flash_device_info->CommandSet.uiSectorEraseAddSeq[0],parallel_flash_device_info->CommandSet.uiSectorEraseDatSeq[0]);
	EP_Send_8bit(parallel_flash_device_info->CommandSet.uiSectorEraseAddSeq[1],parallel_flash_device_info->CommandSet.uiSectorEraseDatSeq[1]);
	EP_Send_8bit(parallel_flash_device_info->CommandSet.uiSectorEraseAddSeq[2],parallel_flash_device_info->CommandSet.uiSectorEraseDatSeq[2]);
	EP_Send_8bit(parallel_flash_device_info->CommandSet.uiSectorEraseAddSeq[3],parallel_flash_device_info->CommandSet.uiSectorEraseDatSeq[3]);
	EP_Send_8bit(parallel_flash_device_info->CommandSet.uiSectorEraseAddSeq[4],parallel_flash_device_info->CommandSet.uiSectorEraseDatSeq[4]);
	EP_Send_8bit(sector_address,parallel_flash_device_info->CommandSet.uiSectorEraseDatSeq[5]);
	
	
	//DQ7 polling mechanism to check if the erase is complete - common across different FLASH devices 
	while(1)
	{
		ucTemp=EP_Receive_8bit(sector_address);	//Read status
	
		if((ucTemp&0x80)==0x80)					//Check if DQ7 = "1", Erase done if "1"  
			return 1;		
		
		if((ucTemp&0x20)==0)					//Check if DQ5 = "0", Skip the next steps and continue the loop if "0"
			continue;
		
		ucTemp=EP_Receive_8bit(sector_address); //Read status
		
		if((ucTemp&0x80)==0x80)					//Check if DQ7 = "1", Erase done if "1"  else report error
			return 1;
			
		return 0;
	}
}


/**************************************************************************************************************************
	Function name	:   Parallel_FLASH_Read_Byte
    Description		:   This function reads a single byte from a particular FLASH address
	Arguments		:	Parameter			 |	Description		 								| Valid values
						uiAddress			 |  Address of the byte to be read from 			| unsigned int 32 bit
	Return value	:   Byte read from the address
**************************************************************************************************************************/
unsigned char  Parallel_FLASH_Read_Byte(unsigned int uiAddress)
{
	unsigned char ucTemp;
	
	ucTemp = EP_Receive_8bit(uiAddress);	//Send EP 8 Bit Receive Command
	
	return ucTemp; 
}


/**************************************************************************************************************************
	Function name	:   Parrallel_FLASH_Read_Bytes
    Description		:   This function reads a series of bytes from a particular FLASH starting location
	Arguments		:	Parameter			 |	Description		 										| Valid values
						ucBuff				 |  Start address of the buffer to be readinto 				| unsigned int 32 bit
						uiStartAddress		 |  Start byte address of the FLASH location		 		| unsigned int 32 bit
						uiLength			 |  Total number of bytes to be read					    | unsigned int 32 bit
	Return value	:   None
**************************************************************************************************************************/
void Parallel_FLASH_Read_Bytes(unsigned char* ucBuff, unsigned int uiStartAddress, unsigned int uiLength)
{

	//Temporary variables
	int i;
	unsigned char ucTemp;
	unsigned char* ucpTemp;
	
	//Initialize the temp pointer to the start address of the internal memory buffer
	ucpTemp=ucBuff;
	
	//Call EP_Receive_8bit function uiLength times
	for(i=0;i<uiLength;i++)
	{
		ucTemp=EP_Receive_8bit(uiStartAddress+i);
		
		*ucpTemp++=ucTemp;
			
	}
}


/**************************************************************************************************************************
	Function name	:   Parallel_FLASH_Byte_Program
    Description		:   This function programs a page of the FLASH
	Arguments		:	Parameter			 		 |	Description		 								| Valid values
						uiAddress					 |  Byte offset 									| unsigned int 32 bit
						ucData						 |  Data to be progammed							| unsigned char 8 bit
						parallel_flash_device_info   |  Struture containing the FLASH device information| Pointer to device info
	Return value	:  =0 for fail, = 1 for pass
**************************************************************************************************************************/
int Parallel_FLASH_Byte_Program(unsigned int uiAddress, unsigned char ucData, PARALLEL_FLASH_DEVICE_INFO* parallel_flash_device_info)
{

	//Temporary variable
	unsigned char ucTemp;
	
	//Send Byte Program command
	EP_Send_8bit(parallel_flash_device_info->CommandSet.uiByteProgramAddSeq[0],
				 parallel_flash_device_info->CommandSet.uiByteProgramDatSeq[0]);
	EP_Send_8bit(parallel_flash_device_info->CommandSet.uiByteProgramAddSeq[1],
				 parallel_flash_device_info->CommandSet.uiByteProgramDatSeq[1]);
	EP_Send_8bit(parallel_flash_device_info->CommandSet.uiByteProgramAddSeq[2],
				 parallel_flash_device_info->CommandSet.uiByteProgramDatSeq[2]);
	EP_Send_8bit(uiAddress,ucData);
	
	
	//DQ7 polling mechanism to check if the program is complete - common across different FLASH devices 
	while(1)
	{
		ucTemp=EP_Receive_8bit(uiAddress);	//Read status		
	
		if((ucTemp&0x80)==(ucData&0x80))	//Check if DQ7 = "DQ7 bit programmed", program done if yes
			return 1;
		
		if((ucTemp&0x20)==0)				//Check if DQ5 = "0", Skip the next steps and continue the loop if "0"
			continue;
		
		ucTemp=EP_Receive_8bit(uiAddress);	//Read status
		
		if((ucTemp&0x80)==(ucData&0x80))	//Check if DQ7 = "DQ7 bit programmed", program done if yes else report error
			return 1;
			
		return 0;
		
	}

}

/**************************************************************************************************************************
	Function name	:   Parallel_FLASH_Write_Byte
    Description		:   This function writes a byte to a particular FLASH location
	Arguments		:	Parameter					 |	Description		 								| Valid values
						uiAddress					 |  Address of the byte location to be writte 		| unsigned int 32 bit
						parallel_flash_device_info   |  Struture containing the FLASH device information| Pointer to device info
						ucData						 |  Data to be written								| unsigned char 8 bit
	Return value	:   None
**************************************************************************************************************************/
void Parallel_FLASH_Write_Byte(unsigned int uiAddress, unsigned char ucData, PARALLEL_FLASH_DEVICE_INFO* parallel_flash_device_info)
{
	Parallel_FLASH_Byte_Program(uiAddress, ucData, parallel_flash_device_info );//Call Parallel FLASH Byte Program function

}

/**************************************************************************************************************************
	Function name	:   Parallel_FLASH_Write_Bytes
    Description		:   This function writes a series of bytes to a particular FLASH location
	Arguments		:	Parameter					 |	Description		 								| Valid values
						ucBuff						 |  Start address of the buffer to be written 		| unsigned int 32 bit
						uiStartAddress				 |  Start byte address of the FLASH location		| unsigned int 32 bit
						uiLength					 |  Total number of bytes to be written			    | unsigned int 32 bit
						parallel_flash_device_info   |  Struture containing the FLASH device information| Pointer to device info
Return value	:   	None
**************************************************************************************************************************/
void Parallel_FLASH_Write_Bytes(unsigned char* ucBuff, unsigned int uiStartAddress, unsigned int uiLength, PARALLEL_FLASH_DEVICE_INFO* parallel_flash_device_info)
{

	//Temporary variables
	int i;
	unsigned char ucTemp;
	unsigned char* ucpTemp;
	
	//Initialize the temp pointer to the start address of the internal memory buffer
	ucpTemp=ucBuff;
	
	//Call Parallel FLASH Byte Program function uiLength times
	for(i=0;i<uiLength;i++)
	{
		ucTemp=*ucpTemp++;
			
		Parallel_FLASH_Byte_Program(uiStartAddress+i, ucTemp, parallel_flash_device_info);
	}

}


/**************************************************************************************************************************
	Function name	:   Program_LDR_File_to_Parallel_FLASH
    Description		:   Programs ldr file contents ASCII format to the parallel FLASH 
	Arguments		:	Parameter				 |	Description		 										| Valid values
						sLDRFilePath			 |  Absolute path of the ldr file							| String
						uiStartAddress			 |  Start byte offset										| Unsigned int 32 bits
						Parallel_FLASH_Info		 |  Pointer to the  FLASH info information					| Unsigned int 32 bits
	Return value	:   -1 if ldr file coudn't be opened, -2 if the verification failed, 1 if everything passed 
**************************************************************************************************************************/
#if !defined(__2137x__)
int Program_LDR_File_to_Parallel_FLASH(char* sLDRFilePath, unsigned int uiStartAddress, PARALLEL_FLASH_DEVICE_INFO *Parallel_FLASH_Info)
{

	//Local variables
	FILE* fp;							//Handle for the ldr file
	unsigned char* ucpTxBuff;			//Pointer to the temporary buffer to store the ldr data	
	unsigned char* ucpVerifyBuff;		//Pointer to the verification buffer
	int i;								//Counter for loops
	unsigned char* ucpTemp;				//Temporary pointers
	unsigned char* ucpTemp1;						
	int iStatus;						//To hold the status of a function
	unsigned char ucTemp;				//Temproary chacter variable
	int iInnerCount;					//Holds the number of words to be progammed for each programming iteration
	int iOuterCount;					//Holds the current value of total bytes already programmed
	int iTemp;							//Temporary int variable
	unsigned int uiTopBlockNumbers;		//Holds the number of top blocks 
	unsigned int uiTopBlockSize;		//Holds the size in bytes of top blocks
	unsigned int uiTopBlockSA;			//Holds the start address of the top block - boundary address between top and bottom blocks
	unsigned int uiBottomBlockNumbers;  //Holds the number of bottom blocks - equals number of sectors in uniform sector case
	unsigned int uiBottomBlockSize;		//Holds the size of bottom blocks - equals size of the sector in uniform sector case

	//Initialize top and bottom block parameters, if the FLASH has non-uniform sectors
	if(Parallel_FLASH_Info->SectorInfo.bUniformSectors==false)
	{
		//If the parameter block is at bottom (starts from zero)
		if(Parallel_FLASH_Info->SectorInfo.bTopParameterBlock==false)
		{
			//Bottom blocks = parameter block
			uiBottomBlockSize		=	Parallel_FLASH_Info->SectorInfo.uiBytesPerParameterBlock;
			uiBottomBlockNumbers 	=  	Parallel_FLASH_Info->SectorInfo.uiTotalParameterBlocks;
			
			//Top blocks = main blocks
			uiTopBlockSize			=	Parallel_FLASH_Info->SectorInfo.uiBytesPerSector;
			uiTopBlockNumbers 		=  	Parallel_FLASH_Info->SectorInfo.uiTotalSectors;
			uiTopBlockSA			=	(Parallel_FLASH_Info->SectorInfo.uiTotalParameterBlocks*Parallel_FLASH_Info->SectorInfo.uiBytesPerParameterBlock);
		}
		//If the parameter block is at top 
		else
		{
			//Bottom block = main blocks
			uiBottomBlockSize		=	Parallel_FLASH_Info->SectorInfo.uiBytesPerSector;
			uiBottomBlockNumbers 	=  	Parallel_FLASH_Info->SectorInfo.uiTotalSectors;
			
			//Top blocks = parameter blocks
			uiTopBlockSize			=	Parallel_FLASH_Info->SectorInfo.uiBytesPerParameterBlock;
			uiTopBlockNumbers 		=  	Parallel_FLASH_Info->SectorInfo.uiTotalParameterBlocks;
			uiTopBlockSA			=	(Parallel_FLASH_Info->SectorInfo.uiTotalSectors*Parallel_FLASH_Info->SectorInfo.uiBytesPerSector);
		}
	}
	//Initialize top and bottom block parameters, if the FLASH has uniform sectors
	else
	{
			uiBottomBlockSize		=	Parallel_FLASH_Info->SectorInfo.uiBytesPerSector;
			uiBottomBlockNumbers 	=  	Parallel_FLASH_Info->SectorInfo.uiTotalSectors;
			uiTopBlockSize			=	0;
			uiTopBlockNumbers 		=  	0;
			uiTopBlockSA			=	0;
	}	
	
	
	//Allocating the temporary buffer size
	ucpTxBuff=(unsigned char*)malloc(LDR_TEMP_BUFF_SIZE*sizeof(*ucpTxBuff));					
	
	//Allocating the temporary buffer size
	ucpVerifyBuff=(unsigned char*)malloc(LDR_TEMP_BUFF_SIZE*sizeof(*ucpVerifyBuff));					
	
	//Open the ldr file in read mode
	fp=fopen(sLDRFilePath, "r");																

	//Return -1 if the file couldn't be opened				
	if(fp==NULL)																				
		return -1;
		
	//Initialize outer count to 0
	iOuterCount=0;
	
	//Read the first byte
	iStatus=fscanf(fp,"%x",&ucTemp);
	
	//Program the ldr file now, continue till end of file
	while(iStatus!=EOF)
	{
		ucpTemp=ucpTxBuff;;	//Reinitialize the pointer to the start of the temp buffer
	
		//Initialize inner count back to 0
		iInnerCount=0; 	
		
		//Initialize the temporary buffer here
		while((iStatus!=EOF)&&(iInnerCount<LDR_TEMP_BUFF_SIZE))
		{
			*ucpTemp++=ucTemp;
			iInnerCount++; 
			iStatus=fscanf(fp,"%x",&ucTemp);
			
		}		
		
		//For first time programming
		if(iOuterCount==0)
		{
			//Erase initial sector 
			Parallel_FLASH_Sector_Erase(uiStartAddress+0, Parallel_FLASH_Info);	
			
			//Uniform sector case
			if(Parallel_FLASH_Info->SectorInfo.bUniformSectors==true)
			{
				if((uiStartAddress/uiBottomBlockSize)!=((uiStartAddress+iInnerCount-1)/uiBottomBlockSize))
				{
					//Erase the next sector if affected
					Parallel_FLASH_Sector_Erase(uiStartAddress+iInnerCount, Parallel_FLASH_Info);	
				}
				
			}
			//Non uniform sector case
			else
			{
				//If start sector is in bottom block
				if(uiStartAddress<uiTopBlockSA)
				{
					//If last byte falls in top block
					if((uiStartAddress+iInnerCount-1)>=uiTopBlockSA)
					{
						//Erase the next sector if affected
						Parallel_FLASH_Sector_Erase(uiStartAddress+iInnerCount-1, Parallel_FLASH_Info);	
					}
					else if((uiStartAddress/uiBottomBlockSize)!=((uiStartAddress+iInnerCount-1)/uiBottomBlockSize))
					{
						//Erase the next sector if affected
						Parallel_FLASH_Sector_Erase(uiStartAddress+iInnerCount-1, Parallel_FLASH_Info);	
					}
				
									
				}	
			
				//Else if the start sector is in top block
				else if((uiStartAddress/uiTopBlockSize)!=((uiStartAddress+iInnerCount-1)/uiTopBlockSize))
				{
					//Erase the next sector if affected
					Parallel_FLASH_Sector_Erase(uiStartAddress+iInnerCount-1, Parallel_FLASH_Info);	
				}
			}
					
			//Write the next iInnterCount bytes using "Parallel FLASH Write Bytes function
			Parallel_FLASH_Write_Bytes(ucpTxBuff, uiStartAddress,iInnerCount, Parallel_FLASH_Info);		
		
			//Read the iInnterCount bytes written above using "Parallel FLASH Read Bytes" function
			Parallel_FLASH_Read_Bytes(ucpVerifyBuff, uiStartAddress,iInnerCount);	
			
			//Compare the buffers for verification
			ucpTemp=ucpTxBuff;
			ucpTemp1=ucpVerifyBuff;
			
			for(i=0;i<iInnerCount;i++)
			{
				if(*ucpTemp!=*ucpTemp1)
					return -2;	//Return -2 if verificaiton failed							
			}
			
			//Increment the outer count by inner count
			iOuterCount+=iInnerCount;
			
			//Check if end of file
			if(iInnerCount!=LDR_TEMP_BUFF_SIZE)				
			{
					//Free the buffers and return if end of file
					free(ucpTxBuff);
					free(ucpVerifyBuff);
					return 1;	
			}
			
		}
		else
		{
			//Uniform sector case	
			if(Parallel_FLASH_Info->SectorInfo.bUniformSectors==true)
			{
				if(((uiStartAddress+iOuterCount-1)/uiBottomBlockSize)!=((uiStartAddress+iOuterCount+iInnerCount-1)/uiBottomBlockSize))
				{
					//Erase the next sector if affected
					Parallel_FLASH_Sector_Erase(uiStartAddress+iOuterCount+iInnerCount-1,Parallel_FLASH_Info);
				}
				
			}
			
			//Non uniform sector case
			else
			{
				//If current sector is in bottom block
				if((uiStartAddress+iOuterCount-1)<uiTopBlockSA)
				{
					//If last byte falls in top block
					if((uiStartAddress+iOuterCount+iInnerCount-1)>=uiTopBlockSA)
					{
						//Erase the next sector if affected
						Parallel_FLASH_Sector_Erase(uiStartAddress+iOuterCount+iInnerCount-1, Parallel_FLASH_Info);	
					}
				
					else if(((uiStartAddress+iOuterCount-1)/uiBottomBlockSize)!=((uiStartAddress+iOuterCount+iInnerCount-1)/uiBottomBlockSize))
					{
						//Erase the next sector if affected
						Parallel_FLASH_Sector_Erase(uiStartAddress+iOuterCount+iInnerCount-1,Parallel_FLASH_Info);	
					}
				
									
				}	
			
				//Else if the current sector is in top block
				else if(((uiStartAddress+iOuterCount-1)/uiTopBlockSize)!=((uiStartAddress+iOuterCount+iInnerCount-1)/uiTopBlockSize))
				{
					//Erase the next sector if affected
					Parallel_FLASH_Sector_Erase(uiStartAddress+iOuterCount+iInnerCount-1,Parallel_FLASH_Info);	
				}
			}
			
			//Write the next iInnterCount bytes using "Parallel FLASH Write Bytes function
			Parallel_FLASH_Write_Bytes(ucpTxBuff, iOuterCount,iInnerCount,Parallel_FLASH_Info);
		
			//Read the iInnterCount bytes written above using "Parallel FLASH Write Bytes function
			Parallel_FLASH_Read_Bytes(ucpVerifyBuff, iOuterCount,iInnerCount);
			
			//Compare the buffers for verification
			ucpTemp=ucpTxBuff;
			ucpTemp1=ucpVerifyBuff;
			
			for(i=0;i<iInnerCount;i++)
			{
				if(*ucpTemp!=*ucpTemp1)
					return -2;								//Return -2 if verificaiton failed
			}
			
			//Increment the outer count by inner count
			iOuterCount+=iInnerCount;
			
			//Check if end of file
			if(iInnerCount!=LDR_TEMP_BUFF_SIZE)			
			{
					//Free the buffers and return if end of file
					free(ucpTxBuff);
					free(ucpVerifyBuff);
					return 1;	
			}
		}
	}
	
	return 1;
}

#endif


/**************************************************************************************************************************
	Function name	:   Program_LDR_File_to_Parallel_FLASH_From_Memory
    Description		:   Programs ldr file contents stored statically in internal/external memory to the parallel FLASH 
	Arguments		:	Parameter				 |	Description		 										| Valid values
						ucpSourceBuff			 |  Start address of the source buffer						| Unsigned int 32 bits
						uiLdrLength				 |  Total size of ldr										| Unsigned int 32 bits
						uiStartAddress			 |  Start byte offset										| Unsigned int 32 bits
						Parallel_FLASH_Info		 |  Pointer to the Serial FLASH info information			| Unsigned int 32 bits
	Return value	:   -1 if ldr file coudn't be opened, -2 if the verification failed, 1 if everything passed 
**************************************************************************************************************************/
int Program_LDR_File_to_Parallel_FLASH_From_Memory(unsigned char* ucpSourceBuff, unsigned int uiLdrLength, unsigned int uiStartAddress, PARALLEL_FLASH_DEVICE_INFO *Parallel_FLASH_Info)
{

	//Local variables
	unsigned char* ucpTxBuff;			//Pointer to the temporary buffer to store the ldr data	
	unsigned char* ucpVerifyBuff;		//Pointer to the verification buffer
	int i;								//For loops
	unsigned char* ucpTemp;				//Temporary char pointers
	unsigned char* ucpTempSource;
	unsigned char* ucpTemp1;	
	int iStatus;						//Status variable
	unsigned char ucTemp;				//Temporary char variable
	int iInnerCount;					//Holds the number of words to be progammed for each programming iteration
	int iOuterCount;					//Holds the current value of total bytes already programmed
	unsigned int uiTopBlockNumbers;		//Holds the number of top blocks 
	unsigned int uiTopBlockSize;		//Holds the size in bytes of top blocks
	unsigned int uiTopBlockSA;			//Holds the start address of the top block - boundary address between top and bottom blocks
	unsigned int uiBottomBlockNumbers;  //Holds the number of bottom blocks - equals number of sectors in uniform sector case
	unsigned int uiBottomBlockSize;		//Holds the size of bottom blocks - equals size of the sector in uniform sector case

	
		
	//Initialize top and bottom block parameters, if the FLASH has non-uniform sectors
	if(Parallel_FLASH_Info->SectorInfo.bUniformSectors==false)
	{
		//If the parameter block is at bottom (starts from zero)
		if(Parallel_FLASH_Info->SectorInfo.bTopParameterBlock==false)
		{
			//Bottom blocks = parameter block
			uiBottomBlockSize		=	Parallel_FLASH_Info->SectorInfo.uiBytesPerParameterBlock;
			uiBottomBlockNumbers 	=  	Parallel_FLASH_Info->SectorInfo.uiTotalParameterBlocks;
			
			//Top blocks = main blocks
			uiTopBlockSize			=	Parallel_FLASH_Info->SectorInfo.uiBytesPerSector;
			uiTopBlockNumbers 		=  	Parallel_FLASH_Info->SectorInfo.uiTotalSectors;
			uiTopBlockSA			=	(Parallel_FLASH_Info->SectorInfo.uiTotalParameterBlocks*Parallel_FLASH_Info->SectorInfo.uiBytesPerParameterBlock);
		}
		//If the parameter block is at top 
		else
		{
			//Bottom block = main blocks
			uiBottomBlockSize		=	Parallel_FLASH_Info->SectorInfo.uiBytesPerSector;
			uiBottomBlockNumbers 	=  	Parallel_FLASH_Info->SectorInfo.uiTotalSectors;
			
			//Top blocks = parameter blocks
			uiTopBlockSize			=	Parallel_FLASH_Info->SectorInfo.uiBytesPerParameterBlock;
			uiTopBlockNumbers 		=  	Parallel_FLASH_Info->SectorInfo.uiTotalParameterBlocks;
			uiTopBlockSA			=	(Parallel_FLASH_Info->SectorInfo.uiTotalSectors*Parallel_FLASH_Info->SectorInfo.uiBytesPerSector);
		}
	}
	//Initialize top and bottom block parameters, if the FLASH has uniform sectors
	else
	{
			uiBottomBlockSize		=	Parallel_FLASH_Info->SectorInfo.uiBytesPerSector;
			uiBottomBlockNumbers 	=  	Parallel_FLASH_Info->SectorInfo.uiTotalSectors;
			uiTopBlockSize			=	0;
			uiTopBlockNumbers 		=  	0;
			uiTopBlockSA			=	0;
	}	
	
	ucpTxBuff=(unsigned char*)malloc(LDR_TEMP_BUFF_SIZE*sizeof(*ucpTxBuff));					//Allocating the temporary buffer size
	
	ucpVerifyBuff=(unsigned char*)malloc(LDR_TEMP_BUFF_SIZE*sizeof(*ucpVerifyBuff));					//Allocating the temporary buffer size
	
	ucpTempSource = ucpSourceBuff;
	
	iOuterCount=0;
	
	//Program the ldr file now
	while(iOuterCount<uiLdrLength)
	{
		ucpTemp=ucpTxBuff;;	//Reinitialize the pointer to the start of the temp buffer
	
		iInnerCount=0; 	
		
		//Initialize the temporary buffer here
		while((iInnerCount<LDR_TEMP_BUFF_SIZE)&&((iOuterCount+iInnerCount)<uiLdrLength))
		{
			*ucpTemp++=*ucpTempSource++;
			iInnerCount++; 
		}		
		
		//For first time programming
		if(iOuterCount==0)
		{
			Parallel_FLASH_Sector_Erase(uiStartAddress, Parallel_FLASH_Info);	//Erase initial sector 
			
			//Uniform sector case
			if(Parallel_FLASH_Info->SectorInfo.bUniformSectors==true)
			{
				if((uiStartAddress/uiBottomBlockSize)!=((uiStartAddress+iInnerCount-1)/uiBottomBlockSize))
				{
					//Erase the next sector if affected
					Parallel_FLASH_Sector_Erase(uiStartAddress+iInnerCount-1, Parallel_FLASH_Info);	
				}
			}
			//Non uniform sector case
			else
			{
				//If start sector is in bottom block
				if(uiStartAddress<uiTopBlockSA)
				{
					//If last byte falls in top block
					if((uiStartAddress+iInnerCount-1)>=uiTopBlockSA)
					{
						//Erase the next sector if affected
						Parallel_FLASH_Sector_Erase(uiStartAddress+iInnerCount-1, Parallel_FLASH_Info);	
					}
					else if((uiStartAddress/uiBottomBlockSize)!=((uiStartAddress+iInnerCount-1)/uiBottomBlockSize))
					{
						//Erase the next sector if affected
						Parallel_FLASH_Sector_Erase(uiStartAddress+iInnerCount-1, Parallel_FLASH_Info);	
					}
				
									
				}	
			
				//Else if the start sector is in top block
				else if((uiStartAddress/uiTopBlockSize)!=((uiStartAddress+iInnerCount-1)/uiTopBlockSize))
				{
					//Erase the next sector if affected
					Parallel_FLASH_Sector_Erase(uiStartAddress+iInnerCount-1, Parallel_FLASH_Info);	
				}
			}
			
			//Write the next iInnterCount bytes using "Parallel FLASH Write Bytes function
			Parallel_FLASH_Write_Bytes(ucpTxBuff, iOuterCount+uiStartAddress,iInnerCount, Parallel_FLASH_Info);	
		
			//Read the iInnterCount bytes written above using "Parallel FLASH Write Bytes function
			Parallel_FLASH_Read_Bytes(ucpVerifyBuff, iOuterCount+uiStartAddress,iInnerCount);	
			
			//Compare the buffers for verification
			ucpTemp=ucpTxBuff;
			ucpTemp1=ucpVerifyBuff;
			
			for(i=0;i<iInnerCount;i++)
			{
				if(*ucpTemp!=*ucpTemp1)
					return -2;								//Return -2 if verificaiton failed
			}
			
			//Increment the outer count by inner count
			iOuterCount+=iInnerCount;
						
			//Check if end of file
			if(iInnerCount!=LDR_TEMP_BUFF_SIZE)				
			{
					//Free the buffers and return if end of file
					free(ucpTxBuff);
					free(ucpVerifyBuff);
					return 1;	
			}
			
		}
		else
		{
		
			//Uniform sector case	
			if(Parallel_FLASH_Info->SectorInfo.bUniformSectors==true)
			{
				if(((uiStartAddress+iOuterCount-1)/uiBottomBlockSize)!=((uiStartAddress+iOuterCount+iInnerCount-1)/uiBottomBlockSize))
				{
					//Erase the next sector if affected
					Parallel_FLASH_Sector_Erase(uiStartAddress+iOuterCount+iInnerCount-1,Parallel_FLASH_Info);
				}
			}
			
			//Non uniform sector case
			else
			{
				//If current sector is in bottom block
				if((uiStartAddress+iOuterCount-1)<uiTopBlockSA)
				{
					//If last byte is in top block
					if((uiStartAddress+iOuterCount+iInnerCount-1)>=uiTopBlockSA)
					{
						//Erase the next sector if affected
						Parallel_FLASH_Sector_Erase(uiStartAddress+iOuterCount+iInnerCount-1, Parallel_FLASH_Info);	
					}
				
					else if(((uiStartAddress+iOuterCount-1)/uiBottomBlockSize)!=((uiStartAddress+iOuterCount+iInnerCount-1)/uiBottomBlockSize))
					{
						//Erase the next sector if affected
						Parallel_FLASH_Sector_Erase(uiStartAddress+iOuterCount+iInnerCount-1,Parallel_FLASH_Info);	
					}
				}	
			
				//Else if the current sector is in top block
				else if(((uiStartAddress+iOuterCount-1)/uiTopBlockSize)!=((uiStartAddress+iOuterCount+iInnerCount-1)/uiTopBlockSize))
				{
					//Erase the next sector if affected
					Parallel_FLASH_Sector_Erase(uiStartAddress+iOuterCount+iInnerCount-1,Parallel_FLASH_Info);	
				}
			}
			
			//Write the next iInnterCount bytes using "Parallel FLASH Write Bytes function
			Parallel_FLASH_Write_Bytes(ucpTxBuff, iOuterCount,iInnerCount, Parallel_FLASH_Info);
		
			//Read the iInnterCount bytes written above using "Parallel FLASH Write Bytes function
			Parallel_FLASH_Read_Bytes(ucpVerifyBuff, iOuterCount,iInnerCount);
			
			//Compare the buffers for verification
			ucpTemp=ucpTxBuff;
			ucpTemp1=ucpVerifyBuff;
			
			for(i=0;i<iInnerCount;i++)
			{
				if(*ucpTemp!=*ucpTemp1)
					return -2;								//Return -2 if verificaiton failed
			}
			
			//Increment the outer count by inner count
			iOuterCount+=iInnerCount;
			
			//Check if end of file
			if(iInnerCount!=LDR_TEMP_BUFF_SIZE)				
			{
					//Free the buffers and return if end of file
					free(ucpTxBuff);
					free(ucpVerifyBuff);
					return 1;	
			}
			
		}
	}
	return 1;
}

/**************************************** END OF FILE*********************************************************************/

